/*
File: ThreadIn.java
Date		Author		Changes
Aug 22 96	Lukasz Beca	Created
*/


package webwisdom.tango.threads;

import java.lang.*;
import java.net.*;
import java.io.*;
import webwisdom.tango.messages.Net;
import webwisdom.tango.messages.Message;
import webwisdom.tango.messages.LogoutMessage;
import webwisdom.tango.threads.ThreadOut;
import webwisdom.tango.consts.Const;
import webwisdom.tango.protocols.ServerProtocol;
import webwisdom.tango.tests.Info;

public class ThreadIn extends ServerProtocol implements Runnable {

  

  private boolean exit;
  private Thread th;   //reference to this thread 
 
  public ThreadIn(ComThread cparam, DbThread dparam, ThreadOut outparam, Socket s, Timeout t) {
    
    super(outparam, cparam, dparam, t);   
    n = new Net(s);    
    th = new Thread(this);
    exit = false;
    th.start();
  }
  
  public void run() {

    Message m;
    int type;
    Info info = new Info(c, d);

    System.out.println("ThreadIn: Input thread started");
  
    while(!exit) {

      try {
	m = n.receive();
      } catch(IOException e) {

	 System.out.println("ThreadIn: Connection broken:" + e);
	e.printStackTrace();

	 if (UID != 0) {
	   
	    System.out.println("ThreadIn: Trying to logout user " + UID);
	    logout();
	    
	 }

	 m = null;
	 stop();	 
	 break;
      }
	 
	
      if (m != null) {
	
	type = m.getType();
     

      if (type == Const.EVENT) 
	
	handleEVENT(m);

      else {


	switch(type) {

	  case Const.LOGIN:
	   handleLOGIN(m);
	   break;

          case Const.LOGOUT:
	   handleLOGOUT(m);
	   stop(); //tests
	   break;
	   
          case Const.JOIN:
	   handleJOIN(m);
	   break;

	  case Const.JANS:
	   handleJANS(m);
	   break;

          case Const.FINISH:
	   handleFINISH(m);
	   break;
	   
          case Const.MASTER:
	   handleMASTER(m);
	   break;

    	  case Const.MANS:
	   handleMANS(m);
	   break;

	  case Const.RJOIN:
	   handleRJOIN(m);
	   break;

	  case Const.RANS:
	   handleRANS(m);
	   break;  

	  case Const.STARTED:
	   handleSTARTED(m);
	   break;
	   
	  case Const.PING:
	  	handlePING(m);
			break;
		 
		
		case Const.GROUP_EVENT:
			handleGROUP_EVENT(m);
			break;
	
          default:
	   System.out.println("ThreadIn: error: command not implemented");
	}

        info.listApplications();

      }
    }
    else {
       System.out.println("ThreadIn: error: invalid message received");
    }

    }  
  }


  public void stop() {

    System.out.println("ThreadIn: thread ended");
    
    exit = true;
           
    out.stop();

    n.close();

  
  }



}


  
